@@ -2,20 +2,29 @@ from __future__ import annotations
 
 import importlib.util
 import inspect
-import os
 import re
 import sys
 import types
 import warnings
 from pathlib import Path
+from typing import TYPE_CHECKING, Any, Literal, overload
 
-from .. import config, console, constants, logger
-from ..scene.scene_file_writer import SceneFileWriter
+from manim._config import config, console, logger
+from manim.constants import (
+    CHOOSE_NUMBER_MESSAGE,
+    INVALID_NUMBER_MESSAGE,
+    NO_SCENE_MESSAGE,
+    SCENE_NOT_FOUND_MESSAGE,
+)
+from manim.scene.scene_file_writer import SceneFileWriter
+
+if TYPE_CHECKING:
+    from manim.scene.scene import Scene
 
 __all__ = ["scene_classes_from_file"]
 
 
-def get_module(file_name: Path):
+def get_module(file_name: Path) -> types.ModuleType:
     if str(file_name) == "-":
         module = types.ModuleType("input_scenes")
         logger.info(
@@ -48,19 +57,22 @@ def get_module(file_name: Path):
             )
 
             spec = importlib.util.spec_from_file_location(module_name, file_name)
-            module = importlib.util.module_from_spec(spec)
-            sys.modules[module_name] = module
-            sys.path.insert(0, str(file_name.parent.absolute()))
-            spec.loader.exec_module(module)
-            return module
+            if isinstance(spec, importlib.machinery.ModuleSpec):
+                module = importlib.util.module_from_spec(spec)
+                sys.modules[module_name] = module
+                sys.path.insert(0, str(file_name.parent.absolute()))
+                assert spec.loader
+                spec.loader.exec_module(module)
+                return module
+            raise FileNotFoundError(f"{file_name} not found")
         else:
             raise FileNotFoundError(f"{file_name} not found")
 
 
-def get_scene_classes_from_module(module):
+def get_scene_classes_from_module(module: types.ModuleType) -> list[type[Scene]]:
     from ..scene.scene import Scene
 
-    def is_child_scene(obj, module):
+    def is_child_scene(obj: Any, module: types.ModuleType) -> bool:
         return (
             inspect.isclass(obj)
             and issubclass(obj, Scene)
@@ -74,9 +86,9 @@ def get_scene_classes_from_module(module):
     ]
 
 
-def get_scenes_to_render(scene_classes):
+def get_scenes_to_render(scene_classes: list[type[Scene]]) -> list[type[Scene]]:
     if not scene_classes:
-        logger.error(constants.NO_SCENE_MESSAGE)
+        logger.error(NO_SCENE_MESSAGE)
         return []
     if config["write_all"]:
         return scene_classes
@@ -89,7 +101,7 @@ def get_scenes_to_render(scene_classes):
                 found = True
                 break
         if not found and (scene_name != ""):
-            logger.error(constants.SCENE_NOT_FOUND_MESSAGE.format(scene_name))
+            logger.error(SCENE_NOT_FOUND_MESSAGE.format(scene_name))
     if result:
         return result
     if len(scene_classes) == 1:
@@ -98,7 +110,7 @@ def get_scenes_to_render(scene_classes):
     return prompt_user_for_choice(scene_classes)
 
 
-def prompt_user_for_choice(scene_classes):
+def prompt_user_for_choice(scene_classes: list[type[Scene]]) -> list[type[Scene]]:
     num_to_class = {}
     SceneFileWriter.force_output_as_scene_name = True
     for count, scene_class in enumerate(scene_classes, 1):
@@ -107,16 +119,22 @@ def prompt_user_for_choice(scene_classes):
         num_to_class[count] = scene_class
     try:
         user_input = console.input(
-            f"[log.message] {constants.CHOOSE_NUMBER_MESSAGE} [/log.message]",
+            f"[log.message] {CHOOSE_NUMBER_MESSAGE} [/log.message]",
         )
-        scene_classes = [
-            num_to_class[int(num_str)]
-            for num_str in re.split(r"\s*,\s*", user_input.strip())
+
+        if user_input == "*":
+            selected_scenes_classes = scene_classes
+        else:
+            selected_scenes_classes = [
+                num_to_class[int(num_str)]
+                for num_str in re.split(r"\s*,\s*", user_input.strip())
+            ]
+        config["scene_names"] = [
+            scene_class.__name__ for scene_class in selected_scenes_classes
         ]
-        config["scene_names"] = [scene_class.__name__ for scene_class in scene_classes]
-        return scene_classes
+        return selected_scenes_classes
     except KeyError:
-        logger.error(constants.INVALID_NUMBER_MESSAGE)
+        logger.error(INVALID_NUMBER_MESSAGE)
         sys.exit(2)
     except EOFError:
         sys.exit(1)
@@ -125,9 +143,31 @@ def prompt_user_for_choice(scene_classes):
         sys.exit(1)
 
 
+@overload
 def scene_classes_from_file(
-    file_path: Path, require_single_scene=False, full_list=False
-):
+    file_path: Path, require_single_scene: bool, full_list: Literal[True]
+) -> list[type[Scene]]: ...
+
+
+@overload
+def scene_classes_from_file(
+    file_path: Path,
+    require_single_scene: Literal[True],
+    full_list: Literal[False] = False,
+) -> type[Scene]: ...
+
+
+@overload
+def scene_classes_from_file(
+    file_path: Path,
+    require_single_scene: Literal[False] = False,
+    full_list: Literal[False] = False,
+) -> list[type[Scene]]: ...
+
+
+def scene_classes_from_file(
+    file_path: Path, require_single_scene: bool = False, full_list: bool = False
+) -> type[Scene] | list[type[Scene]]:
     module = get_module(file_path)
     all_scene_classes = get_scene_classes_from_module(module)
     if full_list:
